"use strict";
/***********************************************************************
 * YoLink CO Smoke Detector device support
 *
 * This will issue warning messages to the HomeBridge log that can then
 * be provided to author to assist in adding device support.
 *
 * Copyright (c) 2023 David Kerr
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.mqttCoSmokeDetector = exports.initCoSmokeDetector = void 0;
/***********************************************************************
 * initUnknownDevice
 *
 */
async function initCoSmokeDetector() {
    var _a, _b, _c;
    const platform = this.platform;
    const accessory = this.accessory;
    const device = this.accessory.context.device;
    if (String(device.config.hide).toLowerCase() === 'co') {
        platform.log.info(`[${device.deviceMsgName}] Hide Carbon Monoxide service because config.[${device.deviceId}].hide is set to "co"`);
        accessory.removeService(accessory.getService(platform.Service.CarbonMonoxideSensor));
    }
    else {
        // Not trying to hide the carbon monoxide service.
        this.coService = accessory.getService(platform.Service.CarbonMonoxideSensor)
            || accessory.addService(platform.Service.CarbonMonoxideSensor);
        this.coService.setCharacteristic(platform.Characteristic.Name, device.name + ' CO');
    }
    if (String(device.config.hide).toLowerCase() === 'smoke') {
        platform.log.info(`[${device.deviceMsgName}] Hide Smoke service because config.[${device.deviceId}].hide is set to "smoke"`);
        accessory.removeService(accessory.getService(platform.Service.SmokeSensor));
    }
    else {
        // Not trying to hide the smoke service.
        this.smokeService = accessory.getService(platform.Service.SmokeSensor)
            || accessory.addService(platform.Service.SmokeSensor);
        this.smokeService.setCharacteristic(platform.Characteristic.Name, device.name + ' Smoke');
    }
    if (device.config.temperature) {
        // If requested add a service for the internal device temperature.
        this.thermoService = accessory.getService(platform.Service.TemperatureSensor)
            || accessory.addService(platform.Service.TemperatureSensor);
        this.thermoService.setCharacteristic(platform.Characteristic.Name, device.name + ' Temperature');
    }
    else {
        // If not requested then remove it if it already exists.
        accessory.removeService(accessory.getService(platform.Service.TemperatureSensor));
    }
    // Call get handler to initialize data fields to current state and set
    // timer to regularly update the data.
    await this.refreshDataTimer(handleGetBlocking.bind(this, 'both'));
    // Once we have initial data, setup all the Homebridge handlers
    (_a = this.coService) === null || _a === void 0 ? void 0 : _a.getCharacteristic(platform.Characteristic.CarbonMonoxideDetected).onGet(handleGet.bind(this, 'co'));
    (_b = this.smokeService) === null || _b === void 0 ? void 0 : _b.getCharacteristic(platform.Characteristic.SmokeDetected).onGet(handleGet.bind(this, 'smoke'));
    (_c = this.thermoService) === null || _c === void 0 ? void 0 : _c.getCharacteristic(platform.Characteristic.CurrentTemperature).onGet(handleGet.bind(this, 'thermo'));
}
exports.initCoSmokeDetector = initCoSmokeDetector;
/***********************************************************************
 * handleGet
 *
 * This is an example of JSON object returned.
 *
 * {
 *   "online": true,
 *   "state": {
 *       "battery": 4,
 *       "devTemperature": 22,
 *       "interval": 120,
 *       "metadata": {
 *           "inspect": true
 *       },
 *       "sche": {
 *           "type": "disable",
 *           "day": 0,
 *           "time": "0:0"
 *       },
 *       "state": {
 *           "unexpected": false,
 *           "sLowBattery": false,
 *           "smokeAlarm": false,
 *           "gasAlarm": false,
 *           "highTempAlarm": false,
 *           "silence": false
 *       },
 *       "stateChangedAt": {
 *           "gasAlarm": 1701962206458,
 *           "smokeAlarm": 1701962206458,
 *           "unexpected": 1702128701737
 *       },
 *       "tz": 0,
 *       "version": "0202",
 *       "lastInspection": {
 *           "time": 1702128770316
 *       }
 *   },
 *   "deviceId": "abcdef0123456789",
 *   "reportAt": "2023-12-09T13:32:50.317Z"
 * }
 */
async function handleGet(sensor = 'smoke') {
    var _a, _b, _c, _d, _e, _f, _g;
    // wrapping the semaphone blocking function so that we return to Homebridge immediately
    // even if semaphore not available.
    const platform = this.platform;
    const device = this.accessory.context.device;
    handleGetBlocking.bind(this, sensor)()
        .then((v) => {
        switch (sensor) {
            case 'thermo':
                this.thermoService.updateCharacteristic(platform.Characteristic.CurrentTemperature, v);
                break;
            case 'co':
                this.coService.updateCharacteristic(platform.Characteristic.CarbonMonoxideDetected, v);
                break;
            default:
                this.smokeService.updateCharacteristic(platform.Characteristic.SmokeDetected, v);
                break;
        }
    });
    // Return current state of the device pending completion of the blocking function
    return ((sensor === 'co')
        ? (((_b = (_a = device.data) === null || _a === void 0 ? void 0 : _a.state) === null || _b === void 0 ? void 0 : _b.state.gasAlarm) ? 1 : 0)
        : (sensor === 'thermo')
            ? ((_e = (_d = (_c = device.data) === null || _c === void 0 ? void 0 : _c.state) === null || _d === void 0 ? void 0 : _d.devTemperature) !== null && _e !== void 0 ? _e : -270)
            : (((_g = (_f = device.data) === null || _f === void 0 ? void 0 : _f.state) === null || _g === void 0 ? void 0 : _g.state.smokeAlarm) ? 1 : 0));
}
async function handleGetBlocking(sensor = 'smoke') {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const platform = this.platform;
    const device = this.accessory.context.device;
    // serialize access to device data.
    const releaseSemaphore = await device.semaphore.acquire();
    // default to normal lebels of CO or Smoke.
    let rc = (sensor === 'thermo') ? -270 : false;
    try {
        if (await this.checkDeviceState(platform, device) && device.data.online) {
            this.logDeviceState(device, `Smoke: ${device.data.state.state.smokeAlarm}, CO" ${device.data.state.state.gasAlarm}, ` +
                `Battery: ${device.data.state.battery} (Requested: ${sensor})`);
            (_a = this.coService) === null || _a === void 0 ? void 0 : _a.updateCharacteristic(platform.Characteristic.StatusActive, true);
            (_b = this.coService) === null || _b === void 0 ? void 0 : _b.updateCharacteristic(platform.Characteristic.StatusFault, false);
            (_c = this.smokeService) === null || _c === void 0 ? void 0 : _c.updateCharacteristic(platform.Characteristic.StatusActive, true);
            (_d = this.smokeService) === null || _d === void 0 ? void 0 : _d.updateCharacteristic(platform.Characteristic.StatusFault, false);
            (_e = this.thermoService) === null || _e === void 0 ? void 0 : _e.updateCharacteristic(platform.Characteristic.StatusActive, true);
            (_f = this.thermoService) === null || _f === void 0 ? void 0 : _f.updateCharacteristic(platform.Characteristic.StatusFault, false);
            if (device.data.state.state.sLowBattery) {
                platform.log.warn(`Device ${device.deviceMsgName} reports low battery`);
            }
            switch (sensor) {
                case 'thermo':
                    rc = device.data.state.devTemperature;
                    break;
                case 'co':
                    rc = device.data.state.state.gasAlarm ? 1 : 0;
                    break;
                default:
                    rc = device.data.state.state.smokeAlarm ? 1 : 0;
                    break;
            }
        }
        else {
            platform.log.error(`Device offline or other error for ${device.deviceMsgName}`);
            (_g = this.coService) === null || _g === void 0 ? void 0 : _g.updateCharacteristic(platform.Characteristic.StatusActive, false);
            (_h = this.coService) === null || _h === void 0 ? void 0 : _h.updateCharacteristic(platform.Characteristic.StatusFault, true);
            (_j = this.smokeService) === null || _j === void 0 ? void 0 : _j.updateCharacteristic(platform.Characteristic.StatusActive, false);
            (_k = this.smokeService) === null || _k === void 0 ? void 0 : _k.updateCharacteristic(platform.Characteristic.StatusFault, true);
            (_l = this.thermoService) === null || _l === void 0 ? void 0 : _l.updateCharacteristic(platform.Characteristic.StatusActive, false);
            (_m = this.thermoService) === null || _m === void 0 ? void 0 : _m.updateCharacteristic(platform.Characteristic.StatusFault, true);
        }
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in CoSmokeDetector handleGet' + platform.reportError + msg);
    }
    finally {
        releaseSemaphore();
    }
    // for both smoke and CO we return integer 1 or 0 for true/false.
    return (rc);
}
/***********************************************************************
 * mqttCoSmokeDetector
 * Handle message received from MQTT server.
 *
 * This is an example of JSON object returned.
 * {
 *   "event": "COSmokeSensor.Report",
 *   "time": 1702171916893,
 *   "msgid": "1702171916892",
 *   "data": {
 *       "state": {
 *           "unexpected": false,
 *           "sLowBattery": false,
 *           "smokeAlarm": false,
 *           "gasAlarm": false,
 *           "highTempAlarm": false,
 *           "silence": false
 *       },
 *       "metadata": {
 *           "inspect": false
 *       },
 *       "battery": 4,
 *       "interval": 120,
 *       "version": "0202",
 *       "devTemperature": 19,
 *       "tz": 0,
 *       "sche": {
 *           "type": "disable",
 *           "day": 0,
 *           "time": "0:0"
 *       },
 *       "loraInfo": {
 *           "netId": "010205",
 *           "signal": -34,
 *           "gatewayId": "abcdef0123456789",
 *           "gateways": 1
 *       },
 *       "stateChangedAt": {
 *           "gasAlarm": 1701962206458,
 *           "smokeAlarm": 1701962206458,
 *           "unexpected": 1702128701737
 *       }
 *   },
 *   "deviceId": "abcdef1234567890"
 *}
 */
async function mqttCoSmokeDetector(message) {
    var _a, _b, _c;
    const platform = this.platform;
    const device = this.accessory.context.device;
    // serialize access to device data.
    const releaseSemaphore = await device.semaphore.acquire();
    try {
        device.updateTime = Math.floor(new Date().getTime() / 1000) + device.config.refreshAfter;
        const mqttMessage = `MQTT: ${message.event} for device ${device.deviceMsgName}`;
        const event = message.event.split('.');
        const batteryMsg = (device.hasBattery && message.data.battery) ? `, Battery: ${message.data.battery}` : '';
        switch (event[1]) {
            case 'Alert':
            // falls through
            case 'Report':
                if (!device.data) {
                    // in rare conditions (error conditions returned from YoLink) data object will be undefined or null.
                    platform.log.warn(`Device ${device.deviceMsgName} has no data field, is device offline?`);
                    break;
                }
                // if we received a message then device must be online
                device.data.online = true;
                // Merge received data into existing data object
                Object.assign(device.data.state, message.data);
                if (!message.data.reportAt) {
                    // mqtt data does not include a report time, so merging the objects leaves current
                    // unchanged, update the time string.
                    device.data.reportAt = device.reportAtTime.toISOString();
                }
                this.logDeviceState(device, `Smoke: ${message.data.state.smokeAlarm}, CO" ${message.data.state.gasAlarm}, ` +
                    `${batteryMsg} (MQTT: ${message.event})`);
                (_a = this.coService) === null || _a === void 0 ? void 0 : _a.updateCharacteristic(platform.Characteristic.CarbonMonoxideDetected, message.data.state.gasAlarm ? 1 : 0);
                (_b = this.smokeService) === null || _b === void 0 ? void 0 : _b.updateCharacteristic(platform.Characteristic.SmokeDetected, message.data.state.smokeAlarm ? 1 : 0);
                (_c = this.thermoService) === null || _c === void 0 ? void 0 : _c.updateCharacteristic(platform.Characteristic.CurrentTemperature, message.data.devTemperature);
                if (message.data.state.sLowBattery) {
                    platform.log.warn(`Device ${device.deviceMsgName} reports low battery`);
                }
                break;
            case 'DataRecord':
            // falls through
            case 'setState':
            // falls through
            case 'setSchedule':
                // No equivalent for this in HomeKit
                platform.liteLog(mqttMessage + ' ' + JSON.stringify(message, null, 2));
                break;
            default:
                platform.log.warn(mqttMessage + ' not supported.' + platform.reportError + JSON.stringify(message, null, 2));
        }
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in mqttCoSmokeDetector' + platform.reportError + msg);
    }
    finally {
        releaseSemaphore();
    }
}
exports.mqttCoSmokeDetector = mqttCoSmokeDetector;
//# sourceMappingURL=coSmokeDetector.js.map