"use strict";
/***********************************************************************
 * YoLink Platform Accessory class
 *
 * Copyright (c) 2022-2023 David Kerr
 *
 * Based on https://github.com/homebridge/homebridge-plugin-template
 *
 * An instance of this class is created for each accessory the platform registers.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.YoLinkPlatformAccessory = void 0;
const deviceHandlers_1 = require("./deviceHandlers");
class YoLinkPlatformAccessory {
    constructor(platform, accessory) {
        var _a, _b;
        this.platform = platform;
        this.accessory = accessory;
        this.deviceMsgName = '';
        Error.stackTraceLimit = 100;
        const device = accessory.context.device;
        this.deviceId = device.deviceId;
        this.deviceId2 = '';
        // Special handling if we have two devices attached to the one accessory
        // only known case right now is for binding a garage door sensor with controller
        this.deviceType = (accessory.context.device2) ? 'GarageDoorCombo' : device.type;
        this.initializeDeviceVars(platform, device);
        // Now initialize device, creating the homebridge services as required.
        // If device type exists in our list of supported services...
        if (deviceHandlers_1.initDeviceService[this.deviceType]) {
            // And it is not experimental, or we are allowing experimental...
            if ((!(deviceHandlers_1.deviceFeatures[this.deviceType].experimental || false) || device.config.enableExperimental)) {
                // Then set accessory information...
                this.infoService = this.accessory.getService(platform.Service.AccessoryInformation);
                this.infoService
                    .setCharacteristic(platform.Characteristic.Manufacturer, 'YoLink')
                    .setCharacteristic(platform.Characteristic.Name, device.name)
                    .setCharacteristic(platform.Characteristic.FirmwareRevision, String((_a = device.config) === null || _a === void 0 ? void 0 : _a.version))
                    .setCharacteristic(platform.Characteristic.Model, String((_b = device.modelName) !== null && _b !== void 0 ? _b : 'n/a'))
                    .setCharacteristic(platform.Characteristic.ProductData, `deviceId: ${device.deviceId}`)
                    // YoLink does not return device serial number in the API, use deviceId instead.
                    .setCharacteristic(platform.Characteristic.SerialNumber, device.deviceId);
                this.infoService
                    .getCharacteristic(platform.Characteristic.Identify).onSet(this.handleIdentifySet.bind(this, device));
                // Many YoLink devices are battery powered, so makes sense to include
                // battery level service.  YoLink reports 0..4, we will convert to 0,25,50,75,100 percent
                // deliberately using 'device.type' here.  If 'device2' has battery handle that in the
                // initDeviceService function.
                if (device.hasBattery) {
                    // We use a name here because an accessory might have two batteries (e.g. GarageDoorCombo)
                    device.batteryService = accessory.getService('Battery')
                        || accessory.addService(platform.Service.Battery, 'Battery', 'battery');
                    device.batteryService
                        .setCharacteristic(platform.Characteristic.Name, device.name)
                        .setCharacteristic(platform.Characteristic.ChargingState, platform.api.hap.Characteristic.ChargingState.NOT_CHARGEABLE)
                        .setCharacteristic(platform.Characteristic.BatteryLevel, 0);
                    device.batteryService
                        .getCharacteristic(platform.Characteristic.BatteryLevel)
                        .onGet(this.handleBatteryGet.bind(this, device));
                }
                // And finally call the device specific initialization...
                deviceHandlers_1.initDeviceService[this.deviceType].bind(this)();
            }
            else {
                platform.log.warn(`[${device.deviceMsgName}] Experimental device skipped. Enable experimental devices in config. `
                    + 'Initializing as Unknown device');
                deviceHandlers_1.initDeviceService['Unknown'].bind(this)();
            }
        }
        else {
            // We do not have support for this device yet.
            deviceHandlers_1.initDeviceService['Unknown'].bind(this)();
        }
        return (this);
    }
    /*********************************************************************
     * initializeDeviceVars
     *
     */
    initializeDeviceVars(platform, device) {
        var _a, _b, _c, _d, _e;
        var _f, _g;
        device.data = {};
        device.lastReportAtTime = 0;
        (_a = (_f = device.config).refreshAfter) !== null && _a !== void 0 ? _a : (_f.refreshAfter = platform.config.refreshAfter);
        device.config.enableExperimental = platform.makeBoolean(device.config.enableExperimental, platform.config.enableExperimental);
        device.config.temperature = platform.makeBoolean(device.config.temperature, platform.config.deviceTemperatures);
        (_b = (_g = device.config).powerFailureSensorAs) !== null && _b !== void 0 ? _b : (_g.powerFailureSensorAs = platform.config.powerFailureSensorAs);
        device.hasBattery = (_d = (_c = deviceHandlers_1.deviceFeatures[device.type]) === null || _c === void 0 ? void 0 : _c.hasBattery) !== null && _d !== void 0 ? _d : false;
        // Set updateTime to now, which will ensure retrieving data from YoLink
        // on our first pass through.
        device.updateTime = Math.floor(new Date().getTime() / 1000);
        // set time of last log message to way back when...
        device.reportAtTime = new Date(0);
        // Using the platform semaphore rather than per-device to fix YoLink 000201 errors.
        device.semaphore = platform.yolinkRequestSemaphore;
        // GarageDoor specific...
        (_e = device.timeout) !== null && _e !== void 0 ? _e : (device.timeout = 45);
        // targetState used to track if garage door has been requested to open or close
        device.targetState = '';
    }
    /*********************************************************************
     * checkDeviceState
     * Updates device status object, sending a request to the YoLink API if we
     * have no data yet, or it has been a long time since the data was updated.
     *
     * Calls to this function should be serialized with deviceSemaphore to
     * prevent sending multiple requests for the same data to the server.
     */
    async checkDeviceState(platform, device) {
        var _a, _b, _c;
        try {
            platform.verboseLog(`[${device.deviceMsgName}] checkDeviceState (refresh after ${device.config.refreshAfter} seconds)`);
            const timestamp = Math.floor(new Date().getTime() / 1000);
            if ((device.config.refreshAfter === 0)
                || ((device.config.refreshAfter > 0) && (timestamp >= device.updateTime))) {
                // If we have never retrieved data from the device, or data is older
                // than period we want to allow, then retrieve new data from the device.
                // Else return with data unchanged.
                device.budp = await platform.yolinkAPI.getDeviceState(platform, device);
                if (device.budp) {
                    device.data = device.budp.data;
                    device.updateTime = timestamp + device.config.refreshAfter;
                    // reportAtTime is the earlier of the time stamp on this message, or
                    // or the time reported in the message from YoLink. We use this to
                    // only log (in like mode), when we have an update.
                    const msgTime = new Date(parseInt(device.budp.msgid));
                    const repTime = new Date((_b = (_a = device.data) === null || _a === void 0 ? void 0 : _a.reportAt) !== null && _b !== void 0 ? _b : '9999-12-31');
                    device.reportAtTime = (msgTime < repTime) ? msgTime : repTime;
                    this.updateBatteryInfo.bind(this, device)();
                }
                else {
                    device.data = undefined;
                    platform.log.error(`[${device.deviceMsgName}] checkDeviceState received no data`);
                }
            }
        }
        catch (e) {
            const msg = ((e instanceof Error) ? e.stack : e);
            const yolinkMsg = msg.substring(7, msg.indexOf(')') + 1);
            const errCode = (_c = msg.split('YoLink API error code: ').pop()) === null || _c === void 0 ? void 0 : _c.substring(0, 6);
            if ((errCode === '000201') || (errCode === '010301') || (errCode === '000201')) {
                // "YoLink API error code are rather common, so don't declare a problem
                platform.liteLog(yolinkMsg + ' - retrying');
            }
            else {
                platform.log.warn('Error in checkDeviceState' + platform.reportError + msg);
            }
        }
        return (device.data);
    }
    /***********************************************************************
     * logDeviceState
     *
     */
    logDeviceState(device, msg) {
        // reportAtTime is the earlier of the time stamp on this message, or
        // or the time reported in the message from YoLink. We use this to
        // only log (in lite mode), when we have an update.
        if (device.lastReportAtTime < device.reportAtTime.getTime()) {
            device.lastReportAtTime = device.reportAtTime.getTime();
            this.platform.liteLog(`[${device.deviceMsgName}] At ${device.reportAtTime.toLocaleString()}: Device state updated: ${msg}`);
        }
        else {
            this.platform.liteLog(`[${device.deviceMsgName}] At ${device.reportAtTime.toLocaleString()}: Device state: ${msg}`);
        }
    }
    /*********************************************************************
     * refreshDataTimer
     * We run a timer for each device to update cached data from the YoLink servers.
     * Timer is set such that it will fire when the updateTime arrives and (via the
     * provided callback) will call above checkDeviceState function at just the
     * right time to force call to yolinkAPI.getDeviceState.  All this to optimize
     * performance of user experience.
     */
    async refreshDataTimer(handleGet, dev = 'main') {
        const platform = this.platform;
        const device = (dev === 'main') ? this.accessory.context.device : this.accessory.context.device2;
        await handleGet();
        platform.log.info(`Device initialized: ${device.deviceMsgName}`);
        if ((dev === 'main') && (--platform.initializeCount === 0)) {
            platform.log.info('All YoLink devices initialized');
        }
        else {
            platform.log.warn(`[${device.deviceMsgName}] Decrement initialize count: ${platform.initializeCount}`);
        }
        if (device.config.refreshAfter >= 60) {
            // We don't allow for updates any more frequently than once a minute.
            const nextUpdateIn = Math.max(60, (device.updateTime || 0) - Math.floor(new Date().getTime() / 1000));
            platform.verboseLog(`[${device.deviceMsgName}] Set data refresh timer for to run every ${nextUpdateIn} seconds`);
            setInterval(async () => {
                await handleGet();
            }, nextUpdateIn * 1000);
        }
    }
    /***********************************************************************
     * updateBatteryInfo
     *
     */
    updateBatteryInfo(device) {
        var _a, _b, _c, _d, _e, _f;
        const platform = this.platform;
        let batteryLevel = 0;
        try {
            if (device.hasBattery) {
                // Some devices wrap battery information under a 'state' object.
                // If nothing defined then assume 0%
                batteryLevel = ((_b = (_a = device.data) === null || _a === void 0 ? void 0 : _a.battery) !== null && _b !== void 0 ? _b : (_d = (_c = device.data) === null || _c === void 0 ? void 0 : _c.state) === null || _d === void 0 ? void 0 : _d.battery) * 25; // could be NaN but that is okay
                const msg = `[${device.deviceMsgName}] Battery level: ${batteryLevel}%`;
                if (batteryLevel <= 25) {
                    (_e = device.batteryService) === null || _e === void 0 ? void 0 : _e.updateCharacteristic(platform.Characteristic.StatusLowBattery, platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
                    this.platform.log.warn(msg);
                }
                else {
                    (_f = device.batteryService) === null || _f === void 0 ? void 0 : _f.updateCharacteristic(platform.Characteristic.StatusLowBattery, platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                    this.platform.verboseLog(msg);
                }
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in updateBatteryInfo' + platform.reportError + msg);
        }
        return (batteryLevel);
    }
    /*********************************************************************
     * handleBatteryGet
     *
     */
    async handleBatteryGet(device) {
        // wrapping the semaphone blocking function so that we return to Homebridge immediately
        // even if semaphore not available.
        const platform = this.platform;
        this.handleBatteryGetBlocking.bind(this, device)()
            .then((v) => {
            device.batteryService.updateCharacteristic(platform.Characteristic.BatteryLevel, v);
        });
        // Return current state of the device pending completion of the blocking function
        return (this.updateBatteryInfo.bind(this, device)());
    }
    async handleBatteryGetBlocking(device) {
        const platform = this.platform;
        // serialize access to device data.
        const releaseSemaphore = await device.semaphore.acquire();
        let rc = 0;
        try {
            if (await this.checkDeviceState(platform, device)) {
                rc = this.updateBatteryInfo.bind(this, device)();
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in handleBatteryGet' + platform.reportError + msg);
        }
        finally {
            releaseSemaphore();
        }
        return (rc);
    }
    /*********************************************************************
     * handleIdentifySet
     *
     */
    async handleIdentifySet(device, value) {
        const platform = this.platform;
        // serialize access to device data.
        const releaseSemaphore = await device.semaphore.acquire();
        try {
            platform.log.info(`[${device.deviceMsgName}] YoLink Device: '${value}' (unsupported)`);
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in handleIdentitySet' + platform.reportError + msg);
        }
        finally {
            releaseSemaphore();
        }
    }
    /*********************************************************************
     * mqttMessage
     *
     */
    async mqttMessage(message) {
        var _a, _b;
        const device = this.accessory.context.device;
        const platform = this.platform;
        try {
            if (device.data) {
                // reportAtTime is the earlier of the time stamp on this message, or
                // or the time reported in the message from YoLink. We use this to
                // only log (in like mode), when we have an update.
                const msgTime = new Date(parseInt(message.msgid));
                const repTime = new Date((_b = (_a = message.data) === null || _a === void 0 ? void 0 : _a.reportAt) !== null && _b !== void 0 ? _b : '9999-12-31');
                device.reportAtTime = (msgTime < repTime) ? msgTime : repTime;
                this.updateBatteryInfo.bind(this, device)();
                if (deviceHandlers_1.mqttHandler[this.deviceType]) {
                    deviceHandlers_1.mqttHandler[this.deviceType].bind(this)(message);
                }
                else {
                    deviceHandlers_1.mqttHandler['Unknown'].bind(this)(message);
                }
            }
            else {
                platform.log.warn(`MQTT: ${message.event} for uninitialized device ${device.deviceMsgName}`
                    + platform.reportError + JSON.stringify(message, null, 2));
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in mqttMessage' + platform.reportError + msg);
        }
        return;
    }
}
exports.YoLinkPlatformAccessory = YoLinkPlatformAccessory;
//# sourceMappingURL=platformAccessory.js.map